/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Tom Vandenhove  (tom.vandenhove@ubizen.com)

    based on cmds.c by Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

#include <stdio.h>
#include <time.h>
#include <string.h>

#include "nomad_ctl.h"
#include "nomadJB_data.h"
#include "id3.h"

extern int nomad_isopen;

/*
 * Exit nomadIIctl
 */
quit(argc, argv)
	char *argv[];
{
	if (nomad_isopen) {
		printf("Closing Nomad Jukebox...\n");
		dev_close();
	}
	exit(0);
}

/* 
 * Command Table 
 */
extern struct cmd cmdtab[];
extern struct cmd *getcmd();
struct songlist *songs_root;


#define HELPINDENT (sizeof ("directory"))

/*
 * Help command.
 */
help(int argc, char *argv[])
{
	register struct cmd *c;

	if (argc == 1) {
		register int i, j, w;
		int columns, width = 0, lines;
		extern int NCMDS;

		printf("Commands may be abbreviated.  Commands are:\n\n");
		for (c = cmdtab; c->c_name; c++) {
			int len = strlen(c->c_name);

			if (len > width)
				width = len;
		}
		width = (width + 8) &~ 7;
		columns = 80 / width;
		if (columns == 0)
			columns = 1;
		lines = (NCMDS + columns - 1) / columns;
		for (i = 0; i < lines; i++) {
			for (j = 0; j < columns; j++) {
				c = cmdtab + j * lines + i;
				if (c->c_name)
					printf("%s", c->c_name);
				if (c + lines >= &cmdtab[NCMDS]) {
					printf("\n");
					break;
				}
				w = strlen(c->c_name);
				while (w < width) {
					w = (w + 8) &~ 7;
					putchar('\t');
				}
			}
		}
		return;
	}
	while (--argc > 0) {
		register char *arg;
		arg = *++argv;
		c = getcmd(arg);
		if (c == (struct cmd *)-1)
			printf("?Ambiguous help command %s\n", arg);
		else if (c == (struct cmd *)0)
			printf("?Invalid help command %s\n", arg);
		else
			printf("%-*s\t%s\n", HELPINDENT,
				c->c_name, c->c_help);
	}
}

int status()
{
	if (nomad_isopen)
		printf("Device connected.\n");
	else
		printf("Not connected.\n");
}

int dev_open()
{
	if (nomadJB_open() != 0) {
		printf("Error: Nomad Jukebox couldn't be opened!\n");
		return -1;
	} else { printf("Success: Nomad Jukebox opened!\n"); }
	if (nomadJB_lock() != 0) {
		printf("Error: Nomad Jukebox couldn't be locked!\n");
		return -1;
	}
	//songs_root = (struct songlist *)nomadJB_filelist();
	printf("Nomad Jukebox succesfully opened.\n");
	return 0;
}

int dev_close()
{
	if (nomadJB_close() != 0) {
		printf("Error: Nomad Jukebox couldn't be closed!\n");
		return -1;
	}
	printf("Nomad Jukebox succesfully closed.\n");
	return 0;
}

int model()
{
	struct model_info *model_info = nomadJB_model();

	if ( model_info == NULL ) {
		printf("Error: Couldn't get Nomad Jukebox model information!\n");
		return -1;
	}
	printf("Model    : %s\n",model_info->type);
	printf("Id       : %s\n",model_info->id);
	printf("Firmware : %i.%i\n",model_info->fw_version_major,model_info->fw_version_minor);
	return 0;
}

int dev_clock(int argc, char *argv[])
{
	extern struct tm dclock;
  
	switch (argc) {
		
	case 1:	goto get_clock;
		
	case 2:		  
		if (!strcmp(argv[1],"get")) {			
			goto get_clock;
		}
		else if (!strcmp(argv[1],"set")) {
			goto set_clock;
		}
		else printf("?Invalid command\n");
		break;
	default:
		printf("?Invalid command\n");
		
	}

	return -1;	

 set_clock:
	/* Always set current clock */
	if (nomadJB_set_clock() != 0) {
		printf("Error: Couldn't set Nomad Jukebox device clock!\n");
		return -1;
	}
	printf("Device clock written.\n");
	return 0;

 get_clock:
	if (nomadJB_get_clock() != 0) {
		printf("Error: Couldn't get Nomad Jukebox device clock!\n");
		return -1;
	}

	printf("Device clock: %s",asctime(&dclock));

	return 0;
}

int owner(int argc, char *argv[])
{
	extern string owner_info;
	int i;
	
	switch (argc) {
		
	case 1:	goto get_owner;
		
	case 2:		  
		if (!strcmp(argv[1],"get")) {			
			goto get_owner;
		}
		else printf("?Invalid command\n");
		break;

	default:
		if (!strcmp(argv[1],"set")) {
			goto set_owner;
		}
		else printf("?Invalid command\n");
		break;
	}

	return -1;	

 set_owner:
	strcpy(owner_info,argv[2]);
	for ( i=4; i<=argc; i++ ) {
		//	printf("%i %s\n",i,argv[i-1]);
		strncat(owner_info, " ", 1);
		strncat(owner_info, argv[i-1], strlen(argv[i-1]));
	}
	if (nomadJB_set_owner() != 0) {
		printf("Error: Couldn't set Nomad Jukebox owner information!\n");
		return -1;
	}
	printf("Owner info written.\n");
	return 0;

 get_owner:
	if (nomadJB_get_owner() != 0) {
		printf("Error: Couldn't get Nomad Jukebox owner information!\n");
		return -1;
	}
	printf("Owner: %s\n",owner_info);
	return 0;
}

int memory()
{
	extern int mem_total,mem_free;

	if (nomadJB_memory() != 0) {
		printf("Error: Couldn't get Nomad Jukebox memory information!\n");
		return -1;
	}
	printf("Memory Info: Total %dMB ; Used %dMB ; Free %dMB\n",mem_total/1024,(mem_total-mem_free)/1024,mem_free/1024);
	return 0;
}

int count()
{
	//extern int file_count;
	int file_count = 0;
	struct songlist *songs;
	songs_root = (struct songlist *)nomadJB_filelist();
	songs = songs_root;
	while ( songs != NULL ) {
		file_count++;
		songs = songs->next;
	}
	
	printf("# of files: %d\n",file_count);
	return 0;
}

int list()
{
	struct songlist *songs;
	TAG *tags;
	songs_root = (struct songlist *)nomadJB_filelist();
	//printf("returned\n");
	songs = songs_root;
	while ( songs != NULL ) {
		printf("location : %i\n",songs->song->idx);
		tags = songs->song->tags;
		while ( tags != NULL ) {
			printf("\tTAG %s",tags->label);
			if ( tags->type == 0 ) printf("\tSTRING %s\n",tags->string);
			else if ( tags->type == 1 ) printf("\tVALUE  %i\n",tags->data);
			tags = tags->next;
		}
		songs = songs->next;
	}
}

int format()
{
	printf("Are you sure? ");

	if (!yesno()) {
		printf("Flash memory format aborted.\n");
		return 0;
	}

	printf("Please wait...\n");

	if (nomadJB_format_flash() != 0) {
		printf("Error: Couldn't format flash memory!\n");
		return -1;
	}

	printf("Flash memory formatted.\n");
	return 0;
}

int presets(int argc,char *argv[])
{
	unsigned int i,preset;
	extern preset_entry radios[RADIO_PRESETS];

	switch (argc) {
		
	case 1:	
		for (i=0; i<=31; i++)
			printf("Preset #%d\t%d.%d\n",i,(radios+i)->freq,(radios+i)->subfreq);
		break;
		
	case 2:		  
		if (!strcmp(argv[1],"all")) {			
			for (i=0; i<=31; i++)
				printf("Preset #%d\t%d.%d\n",i,(radios+i)->freq,(radios+i)->subfreq);
			return 0;
		}
		else if (!strcmp(argv[1],"read")) {
			
			if (nomadJB_read_presets() != 0) {
				printf("Error: Couldn't get radio presets!\n");
				return -1;
			}
			
			printf("Radio presets loaded.\n");
			return 0;
		}
		else if (!strcmp(argv[1],"write")) {

			if (nomadJB_write_presets() != 0) {
				printf("Error: Couldn't set radio presets!\n");
				return -1;
			}

			printf("Radio presets written.\n");
			return 0;

		}
		else printf("?Invalid command\n");
		break;

	case 5:
		if (!strcmp(argv[1],"set")) {

			preset = atoi(argv[2]);

			if (preset >= 0 && preset < RADIO_PRESETS) {
				(radios+preset)->freq = atoi(argv[3]);
				(radios+preset)->subfreq = atoi(argv[4]);
			}
			else printf("Error: Invalid preset number!\n");
					
		}
		else printf("?Invalid command\n");
		break;

	default:
		printf("?Invalid command\n");
		
	}

	return -1;
}

int download(argc, argv)
     char *argv[];
{
	printf("Sorry!. Work in progress...\n");
	return 0;
}

int upload(int argc, char *argv[])
{
	SONG *root;
	printf("upload argc %i\n",argc);
	switch (argc) {
		
	case 1:
		printf("?Invalid command\n");
		break;
	case 2:	
		//printf("got here %s %s\n",argv[0], argv[1]);
		root = id3song(argv[1]);
		//printf("got here 2\n");
		nomadJB_upload_file(argv[1], root,NULL);
		break;
	default:
		printf("?Invalid command\n");
	}
}

int version()
{
	printf("bedouin-0.1, Copyright 2001 Tom Vandenhove <tom.vandenhove@ubizen.com>\n");
}

int yesno ()
{
  char buf[128];
  int len = 0;
  int c;

  while ((c = getchar ()) != EOF && c != '\n')
    if ((len > 0 && len < 127) || (len == 0 && !isspace (c)))
      buf[len++] = c;
  buf[len] = '\0';

  return (buf[0]=='Y' || buf[0]== 'y');
}
