/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Tom Vandenhove  (tom.vandenhove@ubizen.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

#include <id3.h>
#include <stdio.h>
#include <sys/stat.h>
#include "nomadJB_data.h"
#include "id3file.h"

struct id3genre id3tab[] = {
	{ 0,	"Blues"			},
	{ 1,	"Classic Rock"		},
	{ 2,	"Country"		},
	{ 3,	"Dance"			},
	{ 4,	"Disco"			},
	{ 5,	"Funk"			},
	{ 6,	"Grunge"		},
	{ 7,	"Hip-Hop"		},
	{ 8,	"Jazz"			},
	{ 9,	"Metal"			},
	{ 10,	"New Age"	},
	{ 11,	"Oldies"	},
	{ 12,	"Other"		},
	{ 13,	"Pop"		},
	{ 14,	"R&B"		},
	{ 15,	"Rap"		},
	{ 16,	"Reggae"		},
	{ 17,	"Rock"		},
	{ 18,	"Techno"		},
	{ 19,	"Industrial"		},
	{ 20,	"Alternative"		},
	{ 21,	"Ska"		},
	{ 22,	"Death Metal"		},
	{ 23,	"Pranks"		},
	{ 24,	"Soundtrack"		},
	{ 25,	"Euro-Techno"		},
	{ 26,	"Ambient"		},
	{ 27,	"Trip-Hop"		},
	{ 28,	"Vocal"		},
	{ 29,	"Jazz+Funk"		},
	{ 30,	"Fusion"		},
	{ 31,	"Trance"		},
	{ 32,	"Classical"		},
	{ 33,	"Instrumental"		},
	{ 34,	"Acid"		},
	{ 35,	"House"		},
	{ 36,	"Game"		},
	{ 37,	"Sound Clip"		},
	{ 38,	"Gospel"		},
	{ 39,	"Noise"		},
	{ 40,	"AlternRock"		},
	{ 41,	"Bass"		},
	{ 42,	"Soul"		},
	{ 43,	"Punk"		},
	{ 44,	"Space"		},
	{ 45,	"Meditative"		},
	{ 46,	"Instrumental Pop"		},
	{ 47,	"Instrumental Rock"		},
	{ 48,	"Ethnic"		},
	{ 49,	"Gothic"		},
	{ 50,	"Darkwave"		},
	{ 51,	"Techno-Industrial"		},
	{ 52,	"Electronic"		},
	{ 53,	"Pop-Folk"		},
	{ 54,	"Eurodance"		},
	{ 55,	"Dream"		},
	{ 56,	"Southern Rock"		},
	{ 57,	"Comedy"		},
	{ 58,	"Cult"		},
	{ 59,	"Gangsta"		},
	{ 60,	"Top 40"		},
	{ 61,	"Christian Rap"		},
	{ 62,	"Pop/Funk"		},
	{ 63,	"Jungle"		},
	{ 64,	"Native American"		},
	{ 65,	"Cabaret"		},
	{ 66,	"New Wave"		},
	{ 67,	"Psychadelic"		},
	{ 68,	"Rave"		},
	{ 69,	"Showtunes"		},
	{ 70,	"Trailer"		},
	{ 71,	"Lo-Fi"		},
	{ 72,	"Tribal"		},
	{ 73,	"Acid Punk"		},
	{ 74,	"Acid Jazz"		},
	{ 75,	"Polka"		},
	{ 76,	"Retro"		},
	{ 77,	"Musical"		},
	{ 78,	"Rock & Roll"		},
	{ 79,	"Hard Rock"		},
	{ -1 },
};

extern int debug;

char *gettext_id3frame(ID3Tag *tag, ID3_FrameID id)
{
	char *text;
	ID3Frame *frame;
	//printf("looking for frame\n");
	if ((frame = ID3Tag_FindFrameWithID(tag, id)) != NULL) {
		ID3Field *field;
		if ((field = ID3Frame_GetField(frame, ID3FN_TEXT)) != NULL) {
			char title[1024];
			//ID3Field_GetASCII(field, title, 1024);
			ID3Field_GetASCII(field, title, 1024,0);
			//ID3Field_Delete(field);
			//free(field);
			//printf("Album: %s\n", title);
				//printf("Title: %s\n",field.dummy_);
			text = (char *)malloc(strlen(title)+1);
			strcpy(text,title);
			return text;
		}
		else {
			if (debug) { printf("Didn't get the field\n");}
		}
		
		ID3Frame_Delete (frame);
	}
	else {
		if (debug) { printf("Didn't get the frame\n");}
	}
}

TAG *newtag(BYTE type, char *label, char *string, int data)
{
	TAG *tag_p;

	tag_p = (TAG *)malloc(sizeof(TAG));
	tag_p->type = type;
	tag_p->label = label;
	if ( type == 0 ) { 
		tag_p->string = (char *)malloc(strlen(string)+1);
		strcpy(tag_p->string,string);}
	else if ( type == 1 ) { tag_p->data = data; }
	tag_p->next = NULL;

	return tag_p;
}

SONG *id3song( char *filename )
{
	char *text;
	struct stat *BUF;
	TAG *tag_p, *root;
	ID3Tag *tag;
	SONG *song;

	song = (SONG *)malloc(sizeof(SONG));
	if (debug) { printf("*** Reading %s\n", filename);}
	song->tags = tag_p = newtag(0,"CODEC","MP3",0);

	if ((tag = ID3Tag_New()) != NULL) {
		register struct id3genre *c;
		ID3Frame *frame;
		(void) ID3Tag_Link(tag, filename);
		
		text = gettext_id3frame(tag,ID3FID_TITLE);
		if (debug) { printf("Title: %s\n", text);}
		song->title = text;
		tag_p->next = newtag(0,"TITLE",text,0);
		tag_p = tag_p->next;

		text = gettext_id3frame(tag,ID3FID_ALBUM);
		if (debug) { printf("Album: %s\n", text);}
		song->album = text;
		tag_p->next = newtag(0,"ALBUM",text,0);
		tag_p = tag_p->next;

		text = gettext_id3frame(tag,ID3FID_CONTENTTYPE);
		for (c = id3tab; c->id > -1; c++) {
			if ( c->id == strtol(text+1,NULL,10) ) {
				text = c->text;
				if (debug) { printf("Genre : %s\n",c->text);}
			}
		}
		song->genre = text;
		tag_p->next = newtag(0,"GENRE",text,0);
		tag_p = tag_p->next;

		text = gettext_id3frame(tag,ID3FID_LEADARTIST);
		if (debug) { printf("Artist: %s\n", text);}
		song->artist = text;
		tag_p->next = newtag(0,"ARTIST",text,0);
		tag_p = tag_p->next;

		text = gettext_id3frame(tag,ID3FID_YEAR);
		if (debug) { printf("Year: %s\n", text);}
		song->year = strtol(text,NULL,10);
		tag_p->next = newtag(1,"YEAR",NULL,strtol(text,NULL,10));
		tag_p = tag_p->next;

		text = gettext_id3frame(tag,ID3FID_TRACKNUM);
		if (debug) { printf("Track #: %s\n", text);}
		song->track_nr = strtol(text,NULL,10);
		tag_p->next = newtag(1,"TRACK NUM",NULL,strtol(text,NULL,10));
		tag_p = tag_p->next;
	}
	else {
		if (debug) { printf("Didn't get the tag\n");}
	}
	ID3Tag_Delete(tag);
	
	BUF = malloc(sizeof(struct stat));
	stat(filename,BUF);
	if (debug) { printf("size : %i\n",BUF->st_size);}
	
	tag_p->next = newtag(1,"FILE SIZE",NULL,BUF->st_size);
	song->size = BUF->st_size;
	tag_p = tag_p->next;
	song->length = BUF->st_size*8/128/1000;
	tag_p->next = newtag(1,"LENGTH",NULL,BUF->st_size*8/128/1000);
	tag_p = tag_p->next;
	
	return song;
}

TAG *id3tags( char *filename )
{
	char *text;
	TAG *tag_p, *root;
	SONG song;
	struct stat *BUF;
	
	ID3Tag *tag;
	
	if (debug) { printf("*** Reading %s\n", filename);}
	//exit(0);
	if (debug) { printf("test 1\n");}
	root = tag_p = (TAG *)malloc(sizeof(TAG));
	if (debug) { printf("test 2\n");}
	tag_p->type = 0;
	tag_p->label = "CODEC";
	tag_p->string = "MP3";
	tag_p->next = NULL;

	if ((tag = ID3Tag_New()) != NULL) {
		register struct id3genre *c;
		ID3Frame *frame;
		(void) ID3Tag_Link(tag, filename);

		text = gettext_id3frame(tag,ID3FID_TITLE);
		tag_p->next = (TAG *)malloc(sizeof(TAG));
		tag_p = tag_p->next;
		tag_p->type = 0;
		tag_p->label = "TITLE";
		tag_p->string = (char *)malloc(strlen(text)+1);
		strcpy(tag_p->string,text);
	
		text = gettext_id3frame(tag,ID3FID_ALBUM);
		if (debug) { printf("Album: %s\n", text);}
		tag_p->next = (TAG *)malloc(sizeof(TAG));
		tag_p = tag_p->next;
		tag_p->type = 0;
		tag_p->label = "ALBUM";
		tag_p->string = (char *)malloc(strlen(text)+1);
		strcpy(tag_p->string,text);
		//tag_p->string = text;

		text = gettext_id3frame(tag,ID3FID_CONTENTTYPE);
		for (c = id3tab; c->id > -1; c++) {
			if ( c->id == strtol(text+1,NULL,10) ) {
				text = c->text;
				if (debug) { printf("text : %s\n",c->text);}
			}
		}
		tag_p->next = (TAG *)malloc(sizeof(TAG));
		tag_p = tag_p->next;
		tag_p->type = 0;
		tag_p->label = "GENRE";
		tag_p->string = text;
		
		text = gettext_id3frame(tag,ID3FID_LEADARTIST);
		if (debug) { printf("Artist: %s\n", text);}
		tag_p->next = (TAG *)malloc(sizeof(TAG));
		tag_p = tag_p->next;
		tag_p->type = 0;
		tag_p->label = "ARTIST";
		tag_p->string = (char *)malloc(strlen(text)+1);
		strcpy(tag_p->string,text);
		//tag_p->string = text;
		
		text = gettext_id3frame(tag,ID3FID_YEAR);
		if (debug) { printf("Year: %s\n", text);}
		tag_p->next = (TAG *)malloc(sizeof(TAG));
		tag_p = tag_p->next;
		tag_p->type = 1;
		tag_p->label = "YEAR";
		tag_p->data = strtol(text,NULL,10);
		if (debug) { printf("Year: %s\n", text);}

		text = gettext_id3frame(tag,ID3FID_TRACKNUM);
		if (debug) { printf("Track #: %s\n", text);}
		tag_p->next = (TAG *)malloc(sizeof(TAG));
		tag_p = tag_p->next;
		tag_p->type = 1;
		tag_p->label = "TRACK NUM";
		tag_p->data = strtol(text,NULL,10);
		if (debug) { printf("Track: %s\n", text);}

	}
	else {
		if (debug) { printf("Didn't get the tag\n");}
	}
	/* using delete gives segmentation error */
	//ID3Tag_Delete(tag);
	//free(tag);
		
	//tag_p = tag_p->next = (TAG *)malloc(sizeof(TAG));
	tag_p->next = (TAG *)malloc(sizeof(TAG));
	tag_p = tag_p->next;
	tag_p->type = 1;
	tag_p->label = "FILE SIZE";
	BUF = malloc(sizeof(BUF));
	stat(filename,BUF);
	if (debug) { printf("size : %i\n",BUF->st_size);}
	tag_p->data = BUF->st_size;
	if (debug) { printf("size : %i\n",BUF->st_size);}

	//tag_p = tag_p->next = (TAG *)malloc(sizeof(TAG));
	if (debug) { printf("test %i %i\n",tag_p->data,(int)tag_p->next);}
	tag_p->next = NULL;
	if (debug) { printf("test1 \n");}
	//tag_p->next = (TAG *)malloc(sizeof(TAG));
	if (debug) { printf("test2 %i\n",sizeof(TAG));}
	tag_p->next = (TAG *)malloc(sizeof(TAG));
	//tag_p->next = malloc(20);
	if (debug) { printf("test3 \n");}
	if (debug) { printf("size : %i\n",BUF->st_size);}
	tag_p = tag_p->next;
	if (debug) { printf("size : %i\n",BUF->st_size);}
	tag_p->type = 1;
	if (debug) { printf("size : %i\n",BUF->st_size);}
	tag_p->label = "LENGTH";
	/* FIXME: very simplistic !!! */
	if (debug) { printf("size : %i\n",BUF->st_size);}
	tag_p->data = (BUF->st_size*8/128/1000);
	if (debug) { printf("length : %i\n",tag_p->data);}
	
	tag_p->next = NULL;

	return root;
	//exit (0);
}
