/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Tom Vandenhove  (tom.vandenhove@ubizen.com)

    based on nomadII_data.c by Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

#include <fcntl.h>
#include <sys/stat.h>
#include <stdio.h>
#include <time.h>
#include <memory.h>
#include <unistd.h>

#include "data.h"
#include "nomadJB_data.h"
#include "nomad_low_level.h"
#include "id3.h"

int fd;
int nomad_isopen = 0;
int mem_total = 0, mem_free = 0;
int file_count = 0;
string owner_info;
preset_entry radios[RADIO_PRESETS];
struct tm dclock;
extern int debug;

static void print_buffer(BYTE *buffer,int length)
{
	int i,j;
	if (debug) {printf("buffer %i: \n",length);}
	length--;
	for ( i=0; i<=length; i+=0x10 ) {
		if (debug) {printf("%04x:  ",i);}
		for ( j=0; j<0x10; j++ ) {
			if ( i+j > length ) {
				if (debug) {printf("  ");}
				//return;
			} else 
				if (debug) {printf("%02x",buffer[i+j]);}
			//printf("test bits %i\n",j&i);
			if ( (j & 1) == 1 ) { if (debug) {printf(" ");} }
			//printf("%i %02x ",i+j,buffer[i+j]);
		}
		if (debug) {printf(" ");}
		for ( j=0; j<0x10; j++ ) {
			if ( i+j > length ) {
				if (debug) {printf("\n");}
				return;
			}
			if ( buffer[i+j] > 31 && buffer[i+j] < 127 ) 
				{ if (debug) {printf("%c",buffer[i+j]);} }
			else 
				{ if (debug) {printf(".");} }
		}
		if (debug) {printf("\n");}
	}
	//printf("\n       = ");
	/* for ( i=0; i<=length; i++ ) {
	   if ( buffer[i] > 31 && buffer[i] < 127 ) 
	   printf("%c ",buffer[i]);
	   else 
	   printf(". ");
	   } */
	if (debug) {printf("\n");}
}

int nomadJB_open()
{
	if (debug) {printf("trying to open\n");}
	fd = open("/dev/usb/nomadJB",O_RDWR);
	nomad_isopen = (fd > 0) ? 1 : 0;
	
	if (debug) {printf("opened ?\n");}
	return (fd > 0) ? 0 : -1;
}

int nomadJB_lock()
{
	BYTE buffer[1];
  
	if (nomad_read_command(COMMAND_TAKE_CONTROL,0x00,0x00,0x01,buffer) != 0) {
		if (debug) {printf("take control failed\n");}
		return -1;
	} else {
		if (debug) {printf("take control : succes\n");}
	}
	return 0;
}

int nomadJB_init()
{
	/*if (nomadJB_model(model_info)!=0) {
		printf("model_info");
		return -1;
		}*/
	if (nomadJB_memory()!=0) {
		if (debug) {printf("model_memory");}
		return -1; }
	if (nomadJB_count_files()!=0) {
		if (debug) {printf("model_count");}
		return -1; }
	if (nomad_send_command(0x1A,0x00,0x00) != 0) {
		if (debug) {printf("model_command");}
		return -1; }
	if (nomadJB_get_owner()!=0) {
		if (debug) {printf("model_owner");}
		return -1; }
	if (nomad_send_command(0x1A,0x00,0x00) != 0) {
		if (debug) {printf("model_commad_2");}
		return -1; }
	if (nomadJB_memory()!=0) {
		if (debug) {printf("model_mem_2");}
		return -1; }
	if (nomadJB_memory()!=0) {
		if (debug) {printf("model_mem_3");}
		return -1; }
	return 0;
}

int nomadJB_close()
{
	BYTE buffer[1];
	/* Sanity check */
	if (!nomad_isopen)
		return -1;
	
	if (nomad_read_command(COMMAND_RELEASE_CONTROL,0x00,0x00,0x01,buffer) != 0) {
		if (debug) {printf("release control failed\n");}
		return -1;
	} else {
		if (debug) {printf("release control : succes\n");}
	} 

	if (close(fd) == 0) {
		nomad_isopen = 0;
		return 0;
	} else
		return -1;
} 

struct model_info *nomadJB_model()
{
	BYTE buffer[0x3A];
	int i;
	struct model_info *model_info = malloc(sizeof(struct model_info));
	/* returns device id + minor,major,x + device name */
	if (nomad_write_command(COMMAND_GET_DEVICE_ID,0x00,0x00,0x00,NULL) != 0) {
		if (debug) {printf("command get_device_id failed\n");}
		return -1;
	} else {
		if (debug) {printf("command get_device_id : succes\n");}
	}
	
	if (nomad_read(buffer,0x3A) != 0) {
		if (debug) {printf("read buffer failed\n");}
		return -1;
	} else {
		if (debug) {printf("read buffer : succes\n");}
	}
	
	if (debug) { 
		printf("Model    : %s\n",&buffer[25]);
		printf("ID       : ");
	}
	for ( i=1; i<17; i++ )
		if (debug) {printf("%02x",buffer[i]);}
	if (debug) {
		printf("\n");
		printf("Firmware : %i.%i\n",buffer[20],buffer[19]);
		//print_buffer(buffer, 0x3A);
	}
	model_info->type = malloc(strlen(&buffer[25])+1);
	strcpy(model_info->type,(char *)&buffer[25]);
	model_info->id = malloc(33);
	for ( i=1; i<17; i++ )
		sprintf(model_info->id+(i-1)*2,"%02x",buffer[i]);
	model_info->id[32] = '\0';
	//printf("id = %s\n",model_info->id);
	model_info->fw_version_major = buffer[20];
	model_info->fw_version_minor = buffer[19];
	/* Copy model string */
	//strcpy(model_info,(char *) &buffer[OFFSET_MODEL]);
	return model_info;
}

int nomadJB_get_clock()
  /**************************/
  /* Checked Firmware v2.00 */
  /**************************/
{
        /* CONTROL 0x11 --> DOWNSTREAM (IN) (no data)      */
        /* CONTROL <-- UPSTREAM (IN) (lenght 0x11) (clock) */

	//int f;
        BYTE buffer[0x11];

        /* Requesting owner info */
        if (nomad_read_command(COMMAND_GET_CLOCK,0x00,0x00,0x11,buffer) != 0)
                return -1;
	
	//printf("Date : %i/%0.2i/%0.2i\n",buffer[2]*256+buffer[1],buffer[3],buffer[5]);
	//printf("Time : %0.2i:%0.2i:%0.2i\n",buffer[9],buffer[11],buffer[13]);

	/* Fill date structure with date buffer data */

	memcpy(&dclock.tm_year,&buffer[0x01],sizeof(WORD));
	dclock.tm_year-=1900;
	memcpy(&dclock.tm_mon ,&buffer[0x03],sizeof(WORD));
	dclock.tm_mon--;
	memcpy(&dclock.tm_mday,&buffer[0x05],sizeof(WORD));
	memcpy(&dclock.tm_wday,&buffer[0x07],sizeof(WORD));
	memcpy(&dclock.tm_hour,&buffer[0x09],sizeof(WORD));
	memcpy(&dclock.tm_min ,&buffer[0x0B],sizeof(WORD));
	memcpy(&dclock.tm_sec ,&buffer[0x0D],sizeof(WORD));

        return 0;
}

int nomadJB_set_clock()
  /**************************/
  /* Checked Firmware v2.00 */
  /**************************/
{
        /* CONTROL 0x12 --> DOWNSTREAM (OUT) (lenght 0x10) (clock) */
        /* CONTROL <-- UPSTREAM (OUT)                              */
        /* CONTROL 0xF0 --> DOWNSTREAM (IN)  (ack)                 */
	/* CONTROL <-- UPSTREAM (IN) (lenght 0x01) (00)            */

        BYTE buffer[0x10];
	WORD year_aux,mon_aux;
	time_t local_clock;
	
	/* Get current clock on struct tm dclock */ 

	time(&local_clock);
	localtime_r(&local_clock,&dclock);

	year_aux=1900+dclock.tm_year;
	mon_aux=dclock.tm_mon+1;

	memcpy(&buffer[0x00],&year_aux      ,sizeof(WORD));
	memcpy(&buffer[0x02],&mon_aux       ,sizeof(WORD));
	memcpy(&buffer[0x04],&dclock.tm_mday,sizeof(WORD));
	memcpy(&buffer[0x06],&dclock.tm_wday,sizeof(WORD));
	memcpy(&buffer[0x08],&dclock.tm_hour,sizeof(WORD));
	memcpy(&buffer[0x0A],&dclock.tm_min ,sizeof(WORD));
	memcpy(&buffer[0x0C],&dclock.tm_sec ,sizeof(WORD));

        /* Setting device clock */
        if (nomad_write_command(COMMAND_SET_CLOCK,0x00,0x00,0x10,buffer) != 0)
	   return -1;

	/*
	 * Command result
	 */
	if (nomad_send_command(COMMAND_GET_RESULT,0x00,0x00) != 0)
	   return -1;	

        return 0;
}

int nomadJB_get_owner()
  /**************************/
  /* Checked Firmware v2.00 */
  /**************************/
{
        /* CONTROL 0x1C --> DOWNSTREAM (IN) (no data)   */
        /* CONTROL <-- UPSTREAM (IN) (lenght 0x01) (00) */
        /* BULK --> DOWNSTREAM (IN) (lenght 0x40)       */
        /* BULK <-- UPSTREAM   (IN) (data owner)        */

        BYTE buffer[0x81];          /* For bulk transfer */

        /* Requesting owner info */
        if (nomad_write_command(COMMAND_GET_OWNER,0x00,0x00,0x00,NULL) != 0) {
		if (debug) { printf("get_owner failed\n");}
                return -1;
	}

        if (nomad_read(buffer,0x81) != 0)
                return -1;

        /* Copy owner string */
        strcpy(owner_info,(char *) &buffer[OFFSET_OWNER]);

        return 0;
}

int nomadJB_set_owner()
  /**************************/
  /* Checked Firmware v2.00 */
  /**************************/
{
        /* CONTROL 0x1D --> DOWNSTREAM (OUT) (owner data) */
        /* CONTROL <-- UPSTREAM (OUT) (no data)           */
        /* CONTROL 0xF0 --> DOWNSTREAM (IN)  (ack)        */
	/* CONTROL <-- UPSTREAM (IN) (lenght 0x01) (00)   */

        BYTE buffer[0x80];

        /* Copy owner string */
        strcpy((char *) &buffer,owner_info);

        /* Setting owner info */
        if (nomad_write_command(COMMAND_SET_OWNER,0x00,0x00,0x00,NULL) != 0)
                return -1;

	if (nomad_write(buffer,0x80) != 0) 
		return -1;

	/*
	 * Command result
	 */
	if (nomad_send_command(COMMAND_GET_RESULT,0x00,0x00) != 0)
		return -1;	

        return 0;
}

int nomadJB_memory()
  /**************************/
  /* Checked Firmware v2.00 */
  /**************************/
{
        /* CONTROL 0x0D --> DOWNSTREAM (IN) (no data)            */
        /* CONTROL <-- UPSTREAM (IN) (lenght 0x11) (data memory) */

        BYTE buffer[0x11];
	unsigned int i,j;
	//double d;

        if (nomad_read_command(COMMAND_GET_DISK_SPACE,0x00,0x00,0x11,buffer) != 0)
               return -1;
	
	//print_buffer(buffer,0x11);
        memcpy((void *)&i,&buffer[OFFSET_MEM_TOTAL],sizeof(DWORD));
	memcpy((void *)&j,&buffer[OFFSET_MEM_TOTAL+4],sizeof(DWORD));
	//j = 4294967295;
	//printf("mem : %u %u\n",i,j);
	mem_total = (4294967296*i + j)/1024;

        memcpy((void *)&i,&buffer[OFFSET_MEM_FREE],sizeof(DWORD));
	memcpy((void *)&j,&buffer[OFFSET_MEM_FREE+4],sizeof(DWORD));
	mem_free = (4294967296*i + j)/1024;

        return 0;
}

static TAG *decode_file_buffer(BYTE *buffer)
{
	/* should probably be + 256*256*buffer[3] + 256*256*256*buffer[4] */
	//int len = buffer[1] + 256 * buffer[2];
	int nr_tags = buffer[5] + 256 * buffer[6];
	int i,j,idx,tag_label_size,tag_data_size;
	char *tag_name;
	BYTE *tag_data;
	char tag_type;
	//int count;
	int total;
	//int total,factor;
	TAG *tags,*newtag,*oldtag;
	
	oldtag = NULL;
	tags = NULL;
	//printf("Length : %x\n",len);
	//if ( tags == NULL ) printf("test\n");
	//printf("Tags : %i\n",nr_tags);
	
	idx = 7;
	for (i=0; i<nr_tags; i++) {
		//printf("Tag %i : ",i);
		newtag = (TAG *)malloc(sizeof(TAG));
		//printf("Tags : %i\n",nr_tags);
		if ( tags == NULL ) tags = newtag;
		else oldtag->next = newtag;
		//printf("Tags : %i\n",nr_tags);
		oldtag = newtag;
		newtag->next = NULL;
		//printf("Tag %i : ",i); for (j=0; j<8; j++) { printf("%x ",buffer[idx+j]); } printf("\n");
		/* tag_type = 0 for ASCII, 1 for numeric */
		tag_type = buffer[idx] + 256*buffer[idx+1];
		newtag->type = buffer[idx] + 256*buffer[idx+1];
		tag_label_size = buffer[idx+2] + 256*buffer[idx+3];
		tag_data_size  = buffer[idx+4] + 256*buffer[idx+5];
		//printf("\tLabel size : %i\n\tData size : %i\n",tag_label_size,tag_data_size);
		idx += 8;
		
		tag_name = buffer+idx;
		newtag->label = (char *)malloc(strlen(tag_name)+1);
		strcpy(newtag->label,tag_name);
		//printf("\tName : %s e\n",tag_name);
		
		idx += tag_label_size;
		
		tag_data = buffer+idx;
		if ( tag_type == 0 ) { 
			newtag->string = (char *)malloc(strlen(tag_data)+1);
			strcpy(newtag->string,tag_data);
			//printf("\tString : %s %i\n",tag_data,strlen(tag_data)); 
		}
		else if ( tag_type == 1 ) { 
			total = 0;
			//printf("\t"); 
			for (j=tag_data_size-1; j>=0; j--) { total = total*256 + tag_data[j]; } 
			newtag->data = total;
			//printf(" data = %i\n",total); 
		}
		//else { printf("huh? %c\n",tag_type); }
		idx += tag_data_size;
	}

	/* newtag = tags;
	while (tags) {
	  printf("firsttags %i %s ",tags->type,tags->label);
	  if ( tags->type == 0 )
	    //printf("string\n");
	    printf("string %s\n",tags->string);
	  else 
	    //printf("data\n");
	    printf("data %i\n",tags->data);
	  tags = tags->next;
	} 
	printf("end firsttags\n"); 	
	tags = newtag; */
	
	return tags;
}

void add_song_from_tags(struct songlist **songs, int idx, TAG *tags)
{
  SONG *song;
  //printf("adding song\n");
  song = (SONG *)malloc(sizeof(SONG));
  song->idx = idx;
  song->tags = tags;

  while ( tags != NULL ) {
    //printf("tag label %s\n",tags->label);
    if ( strcmp(tags->label,"TITLE") == 0 ) 
      song->title = tags->string;
				//printf("title : %s\n",tags->string);
				//printf("title\n");
    else if ( strcmp(tags->label,"ARTIST") == 0 ) 
      song->artist = tags->string;
    else if ( strcmp(tags->label,"ALBUM") == 0 ) 
      song->album = tags->string;
    else if ( strcmp(tags->label,"GENRE") == 0 ) 
      song->genre = tags->string;
    else if ( strcmp(tags->label,"LENGTH") == 0 ) 
      song->length = tags->data;
    else if ( strcmp(tags->label,"FILE SIZE") == 0 ) 
      song->size = tags->data;
    tags = tags->next;
  }
  //printf("song title = %s\n",song->title);
  add_song_to_list(songs,song);
  
  //*songs = (struct songlist *)malloc(sizeof(struct songlist));
  //(*songs)->song = song;
  //(*songs)->next = NULL;
}

struct songlist *nomadJB_filelist()
{
	BYTE buffer[512];
	//BYTE buffer2[8] = {0x00, 0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00};
	int val,idx,len,keep_on_going;
	int idx_command;
	DWORD location;
	TAG *tags;
	struct songlist *songs,*oldsong;
	struct songlist **psongs;
	oldsong = NULL;
	songs = NULL;
	psongs = &songs;
	keep_on_going = 1;
	idx_command = COMMAND_FILE_GET_FIRST_IDX;
	while (keep_on_going) {
		
		if (nomad_read_command(idx_command,0x00,0x00,0x09,buffer) != 0) {
			if (debug) { printf("command 07 failed\n");}
		} else {
			//printf("succes\n");
		}
		
		/* if response = 10 cc cc cc cc cc cc cc 08 then end of file list */
		if ( buffer[0] == 0x10 ) { if (debug) { printf("end - break\n");} break; }
		
		idx = 256*buffer[2]+buffer[1];
		val = 256*buffer[4]+buffer[3];
		/* plus next 2 bytes... */
		len = 256*buffer[6]+buffer[5] + 5;
		location = 256*256*val+idx;
		//print_buffer(buffer, 0x09);
		//printf("location : %i\n",256*256*val+idx);
		
		if (nomad_write_command(COMMAND_FILE_GET_HEADER,val,idx,0x00,NULL) != 0) {
			if (debug) { printf("command 09 failed\n");}
		} else {
			//printf("succes\n");
		}
		
		if (nomad_read(buffer,len) != 0) {
			if (debug) { printf("read buffer failed\n");}
		} else {
			//printf("succes\n");
		}
		
		//print_buffer(buffer, len); 
		tags = decode_file_buffer(buffer);
		
		add_song_from_tags(psongs,location,tags);
		psongs = &(*psongs)->next;
		
		idx_command = COMMAND_FILE_GET_NEXT_IDX;
	}
	return songs;
}

struct playlist *decode_pl_buffer(BYTE *buffer)
{
  /* should probably be + 256*256*buffer[3] + 256*256*256*buffer[4] */
	extern struct songlist *songs_root;
	/* int len = buffer[1] + 256 * buffer[2];
	   int pl_id = buffer[5] + 256 * buffer[6] + 256*256 * buffer[7] + 256*256*256 * buffer[8]; */
	int pl_name_size = buffer[9];
	int i,idx,nr_tracks,track_id;
	struct songlist *songs, *newsong,*oldsong;
	char *title = NULL;
	struct playlist *playlist = (struct playlist *)malloc(sizeof(struct playlist));
	playlist->next = NULL;
	playlist->songs = NULL;
	
	if (debug) { printf("Name : %s\n",buffer+11);}
	playlist->name = (char *)malloc(strlen(buffer+11)+1);
	strcpy(playlist->name,buffer+11);
	/* now comes 01 00 | 00 00 00 00 | A B C D | 00 00 00 00 | track-1 (4 bytes) | 00 00 00 00 | track-2... 
	   A+256*B... = number of tracks */
	idx = 11 + pl_name_size + 6;
	nr_tracks = buffer[idx] + 256*buffer[idx+1] + 256*256*buffer[idx+2] + 256*256*256*buffer[idx+3];
	if (debug) { printf("# Tracks : %i\n",nr_tracks);}
	idx += 8;
	oldsong = NULL;
	for (i=0; i<nr_tracks; i++) {
		newsong = (struct songlist *)malloc(sizeof(struct songlist));
		newsong->next = NULL;
		track_id = buffer[idx] + 256*buffer[idx+1] + 256*256*buffer[idx+2] + 256*256*256*buffer[idx+3];
		songs = songs_root;
		while ( songs != NULL ) {
			if ( songs->song->idx == track_id ) {
				newsong->song = songs->song;
				title = songs->song->title;
				break;
			}
			songs = songs->next;
		}
		if (debug) { printf("Track %i : %x %s\n",i,track_id,title);}
		idx += 8;
		if ( playlist->songs == NULL ) playlist->songs = newsong;
		else oldsong->next = newsong;
		oldsong = newsong;
	}
	return playlist;
}

struct playlist *nomadJB_playlist() {
	BYTE buffer[512];
	int val,idx,len,keep_on_going;
	int idx_command;
	struct playlist *playlist = NULL;
	struct playlist *newlist, *oldlist = NULL;

	/* get play lists */
	keep_on_going = 1;
	idx_command = COMMAND_PL_GET_FIRST_IDX;
	while (keep_on_going) {

		if (nomad_read_command(idx_command,0x00,0x00,0x09,buffer) != 0) {
			if (debug) { printf("command 11 failed\n");}
		} else {
			if (debug) { printf("command 11 : succes\n");}
		}

		/* if response = 20 cc cc cc cc cc cc cc 08 then end of file list */
		if ( buffer[0] == 0x20 ) { if (debug) { printf("end - break\n");} break; }

		idx = 256*buffer[2]+buffer[1];
		val = 256*buffer[4]+buffer[3];
		/* plus next 2 bytes... */
		len = 256*buffer[6]+buffer[5] + 5;
		//print_buffer(buffer, 0x09);
		if (debug) { printf("location : %i\n",256*256*val+idx);}
		
		if (nomad_write_command(COMMAND_PL_GET_DETAILS,val,idx,0x00,NULL) != 0) {
			if (debug) { printf("command 14 failed\n");}
		} else {
			if (debug) { printf("command 14 : succes\n");}
		}

		if (nomad_read(buffer,len) != 0) {
			if (debug) { printf("read buffer failed\n");}
		} else {
			if (debug) { printf("succes \n");}
		}

		//print_buffer(buffer, len);
		newlist = decode_pl_buffer(buffer);
		if ( playlist == NULL ) playlist = newlist;
		else oldlist->next = newlist;
		oldlist = newlist;

		idx_command = COMMAND_PL_GET_NEXT_IDX;

		/* end of play lists */
	}
	return playlist;
}


int nomadJB_count_files()
  /**************************/
  /* Checked Firmware v2.00 */
  /**************************/
{
        /* CONTROL 0x01 val 0x01 --> DOWNSTREAM (OUT) (no data)     */
        /* CONTROL <-- UPSTREAM (OUT)                               */
        /* CONTROL 0x01 --> DOWNSTREAM (IN)                         */
        /* CONTROL <-- UPSTREAM (IN)  (length 0x03) (data 00 nn 00) */

        BYTE buffer[3];         /* temporary buffer - upstream data */

        //if (nomad_write_command(COMMAND_FILE_COUNT,0x01,0x00,0x00,NULL) != 0)
        //        return -1;

        //if (nomad_read_command(COMMAND_FILE_COUNT,0x00,0x00,0x03,(BYTE*)&buffer) != 0)
        //        return -1;

        memcpy(&file_count,&buffer[OFFSET_FILE_COUNT],sizeof(WORD));

	/* If is not recently formatted, 1 file is always the name map - we hide it */
	if (file_count) file_count--;

        return 0;
}

int nomadJB_get_file(int i, struct nomadJB_file *f)
  /**************************/
  /* Checked Firmware v2.00 */
  /**************************/
{
	void set_file_name(char *, char *);
	void set_file_ext(char *, char *);

	/* CONTROL 0x02 val 0x01 idx #i --> DOWNSTREAM (IN)                      */
	/* CONTROL                      <-- UPSTREAM (IN) (lenght 0x02) (xx yy)  */
	/* BULK                         --> DOWNSTREAM (IN) (lenght 0x215)       */
	/* BULK                         <-- UPSTREAM   (IN) (data chars & name)  */

	//BYTE status[2] = { 0x00, 0x00 };
	BYTE buffer[0x215];
	//DWORD size;

	/* Requesting file info */
	//if (nomad_read_command(COMMAND_FILE_INFO,0x01,i,0x02,(BYTE*)&status) != 0)
	//	return -1;
	
	if (nomad_read(buffer,0x215) != 0)
		return -1;

	set_file_name((char *)&buffer[OFFSET_FILE_NAME],f->file_name);
	set_file_ext((char *)&buffer[OFFSET_FILE_EXT],f->file_ext);

	/* Copy file size */		
	memcpy(&(f->file_size),&buffer[OFFSET_FILE_SIZE],sizeof(DWORD));

	return 0;
}

int nomadJB_format_flash() 
  /**************************/
  /* Checked Firmware v2.00 */
  /**************************/
{
	/* CONTROL 0x0F val 0x201 --> DOWNSTREAM (IN)   */
	/* CONTROL <-- UPSTREAM (IN) (lenght 0x01) (00) */

  //return nomad_send_command(COMMAND_FORMAT, 0x0201, 0x00);	
	return 0;
}


int nomadJB_read_presets()
  /**************************/
  /* Checked Firmware v2.00 */
  /**************************/
{
        /* CONTROL 0x1E --> DOWNSTREAM (IN)                       */
	/* CONTROL <-- UPSTREAM (IN) (lenght 0x41) (presets data) */

	int i;
	short fm1,fm2;

	BYTE buffer[65];

        /* Sanity check */
	if (!nomad_isopen)
		return -1;

	//nomad_ctl_msg (NOMADII_DIR_IN,COMMAND_GET_RADIO_PRESETS,0x00,0x00,0x41,buffer);

	if (buffer > 0) {

		for (i=0; i<=31; i++) {

			memcpy(&fm1,&buffer[2*i+1],2);
      
			fm2 = fm1 % 100;
			fm1 = fm1 / 100;

			(radios+i)->freq = fm1;
			(radios+i)->subfreq = fm2;
		}
    
		return 0;

	}
	
	return -1;
}

int nomadJB_write_presets()
  /**************************/
  /* Checked Firmware v2.00 */
  /**************************/
{
        /* CONTROL 0x1F --> DOWNSTREAM (IN) (lenght 0x40) (presets data) */
	/* CONTROL <-- UPSTREAM (OUT)                                    */
	/* CONTROL 0xF0 --> DOWNSTREAM (IN)                              */
	/* CONTROL <-- UPSTREAM (IN) (lenght 0x01) (00)                  */

	int i;
	short fm;

	BYTE buffer[64];

	/* Sanity check */
	if (!nomad_isopen)
		return -1;

	for (i=0; i<=31; i++) {

		fm = (radios+i)->freq * 100 + (radios+i)->subfreq;

		memcpy(&buffer[2*i],&fm,2);
      
	}

	//if (nomad_ctl_msg (NOMADII_DIR_OUT,COMMAND_SET_RADIO_PRESETS,0x00,0x00,0x40,buffer)<0)
		return -1;

	/*
	 * Command result
	 */

	if (nomad_send_command(COMMAND_GET_RESULT,0x00,0x00) != 0) {

		/* FIXME: add illegal conditions */

		return -1;	
	}

	return 0;
}

int nomadJB_get_libcount() {
	BYTE buffer[512];
	if (nomad_read_command(COMMAND_GET_LIBCOUNT,0x00,0x00,0x19,buffer) != 0) {
		if (debug) { printf("get libcount failed\n");}
	} else {
		if (debug) { printf("get libcount : succes\n");}
	}
	if (debug) { print_buffer(buffer,0x19);}
	/* FIXME: libcount = 8 bytes */
	return buffer[21];
}

static int nomadJB_set_libcount(int libcount) {
	BYTE buffer[512];
	/* FIXME: libcount = 8 bytes */
	BYTE buffer2[8] = {0x00, 0x00, 0x00, 0x00, libcount, 0x00, 0x00, 0x00};
	if (nomad_write_command(COMMAND_SET_LIBCOUNT,0x00,0x00,0x08,buffer2) != 0) {
		if (debug) { printf("set libcount failed\n");}
	} else {
		if (debug) { printf("set libcount : succes\n");}
	}
	
	if (nomad_read_command(COMMAND_GET_RESULT,0x00,0x00,0x01,buffer) != 0) {
		if (debug) { printf("get result failed\n");}
	} else {
		if (debug) { printf("get result : succes\n");}
	}
	return 0;
}

int nomadJB_increase_libcount() {
	int libcount;
	/* FIXME: only need to set libcount once during session */
	libcount = nomadJB_get_libcount();
	libcount++;
	if (debug) { printf("libcount : %i\n",libcount);}
	nomadJB_set_libcount(libcount);
	return 0;
}

static int create_buffer_from_tags(BYTE *buffer,TAG *tag_p) {
	int counter,nr_tags,i,j,k;
	nr_tags = 0;
	counter = 2;
	i = 0;
	while (tag_p) {
		nr_tags++;
		buffer[counter] = tag_p->type;
		buffer[counter+1] = 0;
		buffer[counter+2] = strlen(tag_p->label)+1;
		buffer[counter+3] = 0;
		if ( tag_p->type == 0 )
			buffer[counter+4] = strlen(tag_p->string)+1;
		else 
			/* is this correct ?? */
			buffer[counter+4] = 4;
		buffer[counter+5] = buffer[counter+6] = buffer[counter+7] = 0;
		counter += 8;
		strncpy(buffer+counter,tag_p->label,strlen(tag_p->label)+1);
		counter += strlen(tag_p->label)+1;
		if ( tag_p->type == 0 ) {
			strncpy(buffer+counter,tag_p->string,strlen(tag_p->string)+1);
			counter += strlen(tag_p->string)+1; }
		else if ( tag_p->type == 1 ) {
			j = tag_p->data;
			if (debug) { printf ("foo %i %i\n",j,i);}
			k = 0;
			while (j > 0) {
				i = j & 255;
				//printf ("bar %i %i\n",j,i);
				buffer[counter+k] = i;
				k++;
				j -= i;
				//printf ("bar %i %i\n",j,i);
				j /= 256;
				//printf ("bar %i %i\n",j,i);
			}
			while ( k < 5 ) {
				buffer[counter+k] = 0;
				k++;
			}
			counter += 4; 
		}
		tag_p = tag_p->next;
	}
	buffer[0] = nr_tags;
	buffer[1] = 0;
	print_buffer(buffer,counter);
	return(counter);
}

static void nomadJB_send_header(SONG *song) {
	BYTE buffer[512];
	BYTE buffer2[4] = {0, 0, 0, 0};
	int counter;
	counter = create_buffer_from_tags(buffer,song->tags);
	
	if (debug) { printf("size : %0x\n",counter);}
	buffer2[0] = counter;
	if (nomad_write_command(COMMAND_FILE_SEND_HEADER,0,0,4,buffer2) != 0) {
		if (debug) { printf("command 14 failed\n");}
	} else {
		if (debug) { printf("command 14 : succes\n");}
	}
	if (nomad_write(buffer,counter) != 0) {
		if (debug) { printf("write buffer failed\n");}
	} else {
		if (debug) { printf("write buffer : succes \n");}
	}
	if (nomad_read(buffer,0x05) != 0) {
		if (debug) { printf("read buffer failed\n");}
	} else {
		if (debug) { printf("read buffer : succes\n");}
	}
	/* buffer[0] == 1 probably means song exists allready */
	print_buffer(buffer, 0x05);
	song->idx = buffer[1] + 256*buffer[2] + 256*256*buffer[3] + 256*256*256*buffer[4];
	printf("idx %i %i %i %i %i\n",buffer[1],buffer[2],buffer[3],buffer[4],song->idx);
}

static void send_data(int bytes, BYTE *buffer) {
	BYTE buffer2[1];
	//char c;
 WAIT:
	if (nomad_read_command(COMMAND_FILE_SEND_DATA,bytes,0,1,buffer2) != 0) {
		if (debug) { printf("send data failed\n");}
	} else {
		//printf("send data : succes\n");
	}
	if (buffer2[0] == 0) { 
		//printf("ready to send\n"); 
	}
	else {  
		/* FIXME: pause a little */
		goto WAIT;
		//printf("don't send !!!\n"); 
	}
	if (nomad_write(buffer,bytes) != 0) {
		if (debug) { printf("write buffer failed\n");}
	} else {
		//printf("write buffer : succes \n");
	}
	if (nomad_read_command(COMMAND_GET_RESULT,0x00,0x00,0x01,buffer) != 0) {
		if (debug) { printf("get result failed\n");}
	} else {
		//printf("get result : succes\n");
	}
	//printf("written %i bytes %i\n",bytes,buffer[0]);
	//c = getchar();
}

static void nomadJB_send_data(char *file, int total, void (*feedback)(int count, int total)) {
	BYTE buffer[0xfe00];
	//BYTE buffer2[1];
	int fd, bytes_read,byte_count,i;
	//char c;
	if (debug) { printf("starting upload %s\n",file);}
	i=0;
	byte_count = 0;
	if ( (fd = open( file, O_RDONLY)) == -1)
		{
			if (debug) { printf("Can't open %s\nn",file);}
			exit(1);
		}
	while ( (bytes_read = read(fd,buffer,0xfe00)) == 0xfe00 ) {
		i++;
		byte_count += bytes_read;
		//printf("writing %i bytes %i\n",bytes_read,i);
		/* error reading file */
		send_data(bytes_read, buffer);
		if ( feedback != NULL ) { feedback(byte_count,total); }
	}
	
	i++;
	byte_count += bytes_read;
	//printf("writing %i bytes %i\n",bytes_read,i);
	send_data(bytes_read, buffer);
	
	if ( feedback != NULL ) { feedback(byte_count,total); }

	if (nomad_read_command(COMMAND_FILE_SEND_COMPLETE,0,0,1,buffer) != 0) {
		if (debug) { printf("send complete failed\n");}
	} else {
		//printf("send complete : succes\n");
	}
	while (buffer[0] != 0) {
		//c = getchar();
		if (nomad_read_command(COMMAND_FILE_SEND_COMPLETE,0,0,1,buffer) != 0) {
			if (debug) { printf("send complete failed\n");}
		} else {
			//printf("send complete : succes\n");
		}
	}
}

void fbtest(int count, int total);
void fbtest(int count, int total) {
	if (debug) { printf("feedback %i/%i %i%%\n",count,total,100*count/total);}
}

int nomadJB_upload_file(char *file, SONG *song, void *feedback) {
	//SONG song;
	if (debug) { printf("starting upload %s\n",file);}
	//root = id3file("/home/tom/mp3/bjrk/debut/crying.mp3");
	//song.tags = root;
	//nomadJB_memory();

	nomadJB_send_header(song);
	if (debug) { printf("idx : %0x\n",(int)song->idx);}
	
	nomadJB_increase_libcount();
	//nomadJB_send_data(file,NULL);
	nomadJB_send_data(file,song->size,feedback);
	if (debug) { printf("finished upload %s\n",file);}
	//nomadJB_memory();
	return 0;
}

void nomadJB_delete_song(SONG *song) {
	BYTE buffer[1];
	int val,idx;
	//char c;
	//s = 2039;
	idx = song->idx & 65535;
	val = (song->idx - idx)/65536;
	//c = getchar();
	if (nomad_read_command(COMMAND_FILE_DELETE,val,idx,1,buffer) != 0) {
		printf("command delete file failed\n");
	} else {
		printf("command delete file : succes\n");
	}
	/* wait-a-bit */
	//c = getchar();
	nomadJB_increase_libcount();
	//c = getchar();
}
