/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Tom Vandenhove  (tom.vandenhove@ubizen.com)

    based on nomadII_data.h by Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

#include <strings.h>

//#include "nomadII.h"

#ifndef NOMADJBDATA_H
#define NOMADJBDATA_H

typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef unsigned long DWORD;
typedef char string[256];
typedef int boolean;

/* NOMADJB Commands */
#define COMMAND_GET_DEVICE_ID      0x01
#define COMMAND_GET_DISK_SPACE     0x04
#define COMMAND_FILE_GET_FIRST_IDX 0x06
#define COMMAND_FILE_GET_NEXT_IDX  0x07
#define COMMAND_FILE_GET_HEADER    0x09
#define COMMAND_FILE_SEND_HEADER   0x0A
#define COMMAND_FILE_DELETE        0x0B
#define COMMAND_FILE_CHANGE_HEADER 0x0C
#define COMMAND_FILE_SEND_COMPLETE 0x0E
#define COMMAND_FILE_SEND_DATA     0x0F
#define COMMAND_FILE_RECEIVE_DATA  0x10
#define COMMAND_PL_GET_FIRST_IDX   0x11
#define COMMAND_PL_GET_NEXT_IDX    0x12
#define COMMAND_PL_GET_DETAILS     0x14
#define COMMAND_PL_CREATE          0x15
#define COMMAND_PL_DELETE          0x16
#define COMMAND_PLAY_SONG          0x1D
#define COMMAND_STOP_PLAY          0x1E
#define COMMAND_1E                 0x1E
#define COMMAND_PAUSE_PLAY         0x1F
#define COMMAND_RESUME_PLAY        0x20
#define COMMAND_GET_PROGRESS       0x22
#define COMMAND_23                 0x23
#define COMMAND_24                 0x24
#define COMMAND_GET_EAX            0x24
#define COMMAND_GET_CLOCK          0x29
#define COMMAND_SET_CLOCK          0x2A
#define COMMAND_TAKE_CONTROL       0x2B
#define COMMAND_RELEASE_CONTROL    0x2C
#define COMMAND_GET_EAX_SIZE       0x3A
#define COMMAND_PL_CHANGE_NAME     0x40
#define COMMAND_SET_OWNER          0x41
#define COMMAND_GET_OWNER          0x42
#define COMMAND_GET_LIBCOUNT       0x43
#define COMMAND_SET_LIBCOUNT       0x44
#define COMMAND_ADD_TO_PL          0x46
#define COMMAND_GET_RESULT         0xF0

/*
#define COMMAND_FILE_COUNT         0x01
#define COMMAND_FILE_INFO          0x02
#define COMMAND_SELECT_FILE        0x03
#define COMMAND_END_TRANSFER       0x04
#define COMMAND_SEND_BLOCK         0x05
#define COMMAND_SONG_TAG           0x06
#define COMMAND_DELETE_FILE        0x08
#define COMMAND_MEMORY             0x0D
#define COMMAND_MODEL              0x0E
#define COMMAND_FORMAT             0x0F
#define COMMAND_PREPARE2           0x10
#define COMMAND_GET_CLOCK          0x11
#define COMMAND_SET_CLOCK          0x12
#define COMMAND_PREPARE            0x1A
#define COMMAND_GET_OWNER          0x1C
#define COMMAND_SET_OWNER          0x1D
#define COMMAND_GET_RADIO_PRESETS  0x1E
#define COMMAND_SET_RADIO_PRESETS  0x1F
#define COMMAND_BEGIN_TRANSFER     0x25
#define COMMAND_GET_RESULT         0xF0
*/
/* NOMAD II Command Result codes */

#define RESULT_ILLEGAL_FREQUENCY      0xD0 /* 208 */
#define RESULT_ILLEGAL_FREQ_DECIMAL   0xD1 /* 209 */

/* NOMADII File Types */

#define FILE_OTHER  0
#define FILE_MP3    1
#define FILE_NVF    2

/* NOMADII Song formats */

#define FORMAT_OTHER 0
#define FORMAT_MP3   1

/* NOMADII Radio Presets */

#define RADIO_PRESETS 32

/* NOMADII Low level byte offsets */

#define OFFSET_MODEL         48

#define OFFSET_OWNER         1

#define OFFSET_FILE_COUNT    1

#define OFFSET_FILE_NAME     513
#define OFFSET_FILE_EXT      521
#define OFFSET_FILE_SIZE     524

#define OFFSET_MEM_TOTAL     1
#define OFFSET_MEM_FREE      9

/* NOMADII Block Size in transfers */

#define BLOCK_SIZE           0xFE00

/* NOMADII file structure */

struct nomadJB_file {
	char file_name[9];
	char file_ext[4];
	DWORD file_size;
};

typedef struct tag_s {
	BYTE type;
	char *label;
	char *string;
	int data;
	struct tag_s *next;
} TAG;

typedef struct song_s {
	DWORD idx;
	char *title;
	char *artist;
	char *album;
	char *genre;
	int year, track_nr, length, size;
	TAG *tags;
} SONG;

struct songlist {
	SONG *song;
	struct songlist *next;
};

struct playlist {
	char *name;
	struct songlist *songs;
	struct playlist *next;
};

struct album {
	char *name;
	struct songlist *songs;
	//struct album *next;
};

struct albumlist {
	struct album *album;
	struct albumlist *next;
};

struct artist {
	char *name;
	struct albumlist *albums;
	struct artist *next;
};

struct genre {
	char *name;
	struct albumlist *albums;
	struct genre *next;
};

struct uploadlist {
	char *file;
	SONG *song;
	struct uploadlist *next;
};

struct model_info {
	char *type;
	char *id;
	int fw_version_major;
	int fw_version_minor;
};

/* NOMADII radio preset entry */

typedef struct {

  short freq,subfreq;

} preset_entry; 

/* NOMADJB Function prototypes */

int nomadJB_open(void);
int nomadJB_lock(void);
int nomadJB_init(void);
int nomadJB_close(void);
struct model_info *nomadJB_model();
int nomadJB_get_clock(void);
int nomadJB_set_clock(void);
int nomadJB_get_owner(void);
int nomadJB_set_owner(void);
int nomadJB_memory(void);
struct songlist *nomadJB_filelist(void);
int nomadJB_count_files(void);
int nomadJB_get_file(int, struct nomadJB_file *);
int nomadJB_format_flash(void);
int nomadJB_read_presets(void);
int nomadJB_write_presets(void);
int nomadJB_get_libcount(void);
int nomadJB_increase_libcount(void);
int nomadJB_upload_file(char *,SONG *, void *);
void add_song_from_tags(struct songlist **songs, int idx, TAG *tags);
struct playlist *nomadJB_playlist(void);
struct playlist *decode_pl_buffer(BYTE *buffer);
void nomadJB_delete_song(SONG *song);

#endif /* NOMADJBDATA_H */



