/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

#include <stdio.h>
#include <signal.h>
#include <string.h>

#include "nomad_ctl.h"
#include "nomadJB_data.h"
#include "callbacks.h"

int     from_a_tty;
char	cmdline[200];
int	margc;
char	*margv[20];
int debug = 0;

int debug;
struct songlist *songs_root;
struct genre *genres_root;
struct artist *artists_root;
struct albumlist *albums_root;
struct playlist *playlist_root;
struct selection selection;

/* 
 * Function prototypes 
 */
struct	cmd *getcmd();
void intr();

int dec_open(),dev_close();

main (argc,argv)
     char *argv[];
{
	register struct cmd *c;
	from_a_tty = isatty(fileno(stdin));

	signal(SIGINT, intr);

	printf("argc %i\n",argc);
	if ( argc > 1 ) {
		printf("argv %s\n",argv[1]);
		c = getcmd(argv[1]);
		if (c == (struct cmd *)-1) {
			printf("?Ambiguous command\n");
			//continue;
			exit(1);
		}
		if (c == 0) {
			printf("?Invalid command\n");
			//continue;
			exit(1);
		}
		dev_open();
		//printf("executing command %s\n",c->c_name);
		argc--; 
		argv++;
		(*c->c_handler)(argc, argv);
		dev_close();
		exit(0);
	}

	for (;;) 
		cmdscanner();
}

/*
 * Signal Handler - SIGINT
 */
void intr()
{
	exit(0);
}

/*
 * Command parser.
 */
cmdscanner()
{
	register struct cmd *c;

	version();

	for (;;) {
		if (from_a_tty) {
			printf("bedouin> ");
			fflush(stdout);
		}
		if (fgets(cmdline, sizeof(cmdline), stdin) == 0)
			quit();
		if (cmdline[0] == 0 || cmdline[0] == '\n')
			continue;
		makeargv();
		c = getcmd(margv[0]);
		if (c == (struct cmd *)-1) {
			printf("?Ambiguous command\n");
			continue;
		}
		if (c == 0) {
			printf("?Invalid command\n");
			continue;
		}
		(*c->c_handler)(margc, margv);
	}
}

/* 
 * Command Table 
 */
extern struct cmd cmdtab[];

struct cmd *getcmd(name)
	register char *name;
{
	register char *p, *q;
	register struct cmd *c, *found;
	register int nmatches, longest;

	longest = 0;
	nmatches = 0;
	found = 0;
	for (c = cmdtab; p = c->c_name; c++) {
		for (q = name; *q == *p++; q++)
			if (*q == 0)		/* exact match? */
				return(c);
		if (!*q) {			/* the name was a prefix */
			if (q - name > longest) {
				longest = q - name;
				nmatches = 1;
				found = c;
			} else if (q - name == longest)
				nmatches++;
		}
	}
	if (nmatches > 1)
		return((struct cmd *)-1);
	return(found);
}

/*
 * Slice a string up into argc/argv.
 */
makeargv()
{
	register char *cp;
	register char **argp = margv;

	margc = 0;
	for (cp = cmdline; *cp;) {
		while (isspace(*cp))
			cp++;
		if (*cp == '\0')
			break;
		*argp++ = cp;
		margc += 1;
		while (*cp != '\0' && !isspace(*cp))
			cp++;
		if (*cp == '\0')
			break;
		*cp++ = '\0';
	}
	*argp++ = 0;
}
