/*  ----------------------------------------------------------------------

    Copyright (C) 2000  Gabriel Gomiz  (gomita@bblanca.com.ar)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    ---------------------------------------------------------------------- */

#include <fcntl.h>
#include <sys/stat.h>
#include <stddef.h>

#include "nomad.h"
#include "types.h"

extern int nomad_isopen;
extern int fd;

/*
 *  Low level commands
 */

int nomad_read_command(int req, int val, int idx, int len, BYTE *buffer)
{
	/* Sanity check */
	if (!nomad_isopen)
		return -1;

	return (nomad_ctl_msg(NOMAD_DIR_IN,req,val,idx,len,buffer) >= 0) ? 0 : -1;
}

int nomad_write_command(int req, int val, int idx, int len, BYTE *buffer)
{
        /* Sanity check */
        if (!nomad_isopen)
                return -1;

        return (nomad_ctl_msg(NOMAD_DIR_OUT,req,val,idx,len,buffer) >= 0) ?
0 : -1;
}

int nomad_read(BYTE *buffer, int count)
{
	/* Sanity check */
	if (!nomad_isopen)
		return -1;

	return (read(fd,buffer,count) == count) ? 0 : -1;	
}

int nomad_write(BYTE *buffer, int count)
{
  int test;
        /* Sanity check */
        if (!nomad_isopen)
                return -1;
	test = write(fd,buffer,count);
	/* printf("Count = %i\n",test); */
        return (test == count) ? 0 : -1;
}

int nomad_send_command(int req, int val, int idx)
{
        BYTE status;

        /* Sanity check */
        if (!nomad_isopen)
                return -1;

        nomad_ctl_msg(NOMAD_DIR_IN,req,val,idx,0x01,&status);

        return status;
}

int nomad_reset_pipes()
{
	struct Nomad_cmd cmd;

	/* Sanity check */
	if (!nomad_isopen)
		return -1;

	cmd.timeout     = 50;
	cmd.requesttype = 0;
	cmd.request     = 0;
	cmd.value       = 0;
	cmd.index       = 0;
	cmd.length      = 0;
	cmd.buffer      = NULL;

	return (ioctl (fd, NOMAD_RESET_PIPES, &cmd));
}

int nomad_ctl_msg(int dir, int req, int val, int idx, int len, BYTE *d)
{
	int result;
	struct Nomad_cmd cmd;

	/* Sanity check */
	if (!nomad_isopen)
		return -1;

	/* Rest of parameters */

	cmd.requesttype = 0;
	cmd.request     = req;
	cmd.value       = val;
	cmd.index       = idx;
	cmd.length      = len;
	cmd.buffer      = d;
	cmd.timeout     = 50;

	switch (dir) {
		
	case NOMAD_DIR_IN: 
		
		result = ioctl (fd, NOMAD_RECV_COMMAND, &cmd);
		break;
			
	case NOMAD_DIR_OUT: 
		
		result = ioctl (fd, NOMAD_SEND_COMMAND, &cmd);
		break;

	default:
		return -1;
	}

	return result;
}

/*
 * Auxiliary functions
 */

void set_file_name(char *src, char *dst) 
{
	int i;

	for (i = 0; i < 8; i++)
		*(dst+i) = *(src+i);
	*(dst+8) = '\0';
}

void set_file_ext(char *src, char *dst) 
{
	int i;
	
	for (i = 0; i < 3; i++)
		*(dst+i) = *(src+i);
	*(dst+3) = '\0';
}
